package com.platform.cac;


import com.fasterxml.jackson.core.type.TypeReference;
import com.platform.info.GlobalData;
import com.platform.info.dto.*;
import com.platform.util.HttpClientUtils;
import com.platform.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.net.ConnectException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.platform.util.BaseAuthorizationUtils.generateAuthAndDateHeader;


@Slf4j
@Component
public class CacHpApi {

    private static final String HTFP_PATH = "/htfp/gcs";
    public static String HOST;
    @Value("${http-cac.host}")
    public void setHost(String host) {
        CacHpApi.HOST = host;
        log.info("[cac-api] >>>中心指控地址 {}<<<", host);
    }

    private static String postRequest(String api, String body)  {
        String path = "http://" + HOST + api;
        HashMap<String, String> header = new HashMap<>(3);
        generateAuthAndDateHeader(header, "POST", GlobalData.GCS_ID, GlobalData.GCS_TOKEN, api);
        try {
            return HttpClientUtils.sendPost(path,  body, header);
        } catch (ConnectException e) {
            log.error("[cac-api] http请求超时: {}", e.getMessage());
            return null;
        } catch (IOException e) {
            log.error("[cac-api] http请求错误: {}", e.getMessage());
            return null;
        }
    }
    private static Result<String> postRequestAndGetResult(String api, String body)  {
        String path = "http://" + HOST + api;
        HashMap<String, String> header = new HashMap<>(3);
        generateAuthAndDateHeader(header, "POST", GlobalData.GCS_ID, GlobalData.GCS_TOKEN, api);
        try {
            String responseStr = HttpClientUtils.sendPost(path, body, header);
            return JSONUtils.json2obj(responseStr, new TypeReference<Result<String>>() {});
        } catch (ConnectException e) {
            log.error("[cac-api] http请求超时: {}", e.getMessage());
            return null;
        } catch (Exception e) {
            log.error("[cac-api] http请求错误: {}, body={}", e.getMessage(), body);
            return null;
        }
    }


    /**
     * 地面站上线
     */
    public static String gcsSignIn(String body) {
        final String gcsSignApi = HTFP_PATH + "/signIn";
        Result<String> result = postRequestAndGetResult(gcsSignApi, body);
        if (result == null || !result.isSuccess()) {
            log.error("[cac-api] 地面站上线失败: {}", result);
            return null;
        }
        return result.getData();
    }

    /**
     * 地面站下线 （哈勃不下线）
     */
    // public static String gcsSignOut(String body) {
    //     final String gcsSignApi = HTFP_PATH + "/signOut";
    //     return postRequest(gcsSignApi, body);
    // }

    /**无人机上线*/
    public static boolean uavMasterControlNotify(String uavId) {
        final String gcsSignApi = HTFP_PATH + "/notifyGetUavControlRight";
        Map<String, String> body = new HashMap<>();
        body.put("uavId", uavId);
        body.put("gcsId", GlobalData.GCS_ID);
        try {
            Result<String> result = postRequestAndGetResult(gcsSignApi, JSONUtils.obj2json(body));
            if (result == null || !result.isSuccess()) {
                log.error("[cac-api] 无人机上线失败: {}", result);
                return false;
            }
            return true;
        } catch (IOException e) {
            log.error("[cac-api] 无人机上线失败: {}", e.getMessage());
        }
        return false;
    }
    /**
     * 无人机下电
     */
    public static boolean uavPowerOff(String uavId) {
        final String gcsSignApi = HTFP_PATH + "/uavPowerOff";
        Map<String, String> body = new HashMap<>();
        body.put("uavId", uavId);
        body.put("gcsId", GlobalData.GCS_ID);
        try {
            Result<String> result = postRequestAndGetResult(gcsSignApi, JSONUtils.obj2json(body));
            if (result == null || !result.isSuccess()) {
                log.error("[cac-api] 无人机下线失败: {}", result);
                return false;
            }
            return true;
        } catch (IOException e) {
            log.error("[cac-api] 无人机下线失败: {}", e.getMessage());
        }
        return false;
    }



    /**
     * 指令执行结果通知
     */
    public static boolean commandNotify(CommandResultParam commandResultParam) throws IOException {
        final String gcsSignApi = HTFP_PATH + "/notifyUavCommand";
        Result<String> result = postRequestAndGetResult(gcsSignApi, JSONUtils.obj2json(commandResultParam));
        if (result == null || !result.isSuccess()) {
            log.error("[cac-api] 指令执行结果通知失败: {}", result);
            return false;
        }
        return true;
    }
    /**
     * 参数查询结果通知
     */
    public static boolean uavParamQueryResultNotify(QueryResultParam queryResultParam) throws IOException {
        final String gcsSignApi = HTFP_PATH + "/uavParamQueryResultNotify";
        Result<String> result = postRequestAndGetResult(gcsSignApi, JSONUtils.obj2json(queryResultParam));
        if (result == null || !result.isSuccess()) {
            log.error("[cac-api] 参数查询结果通知失败: {}", result);
            return false;
        }
        return true;
    }
    /**

     * 装订执行结果通知
     */
    public static boolean uavParamBindResultNotify(BindResultParam bindResultParam) throws IOException {
        final String gcsSignApi = HTFP_PATH + "/uavParamBindResultNotify";
        Result<String> result = postRequestAndGetResult(gcsSignApi, JSONUtils.obj2json(bindResultParam));
        if (result == null || !result.isSuccess()) {
            log.error("[cac-api] 装订执行结果通知失败: {}", result);
            return false;
        }
        return true;
    }



    public static List<DirectControlUavParam> queryAllCacDirectControlUavList(String body) {
        final String gcsSignApi = HTFP_PATH + "/queryAllCacDirectControlUavList";
        try {
            Result<String> result = postRequestAndGetResult(gcsSignApi, body);
            if (result == null || !result.isSuccess()){
                log.error("查询全量无人机映射关系失败: {}", result);
                return null;
            }
            return JSONUtils.json2list(result.getData(), DirectControlUavParam.class);
        } catch (Exception e) {
            log.error("查询全量无人机映射关系失败: {}", e.getMessage());
        }
        return null;
    }

    public static DirectControlUavParam queryCacDirectControlUav(String haborSn) {
        final String gcsSignApi = HTFP_PATH + "/queryCacDirectControlUav";
        Map<String, String> body = new HashMap<>();
        body.put("haborSn", haborSn);
        try {
            Result<String> result = postRequestAndGetResult(gcsSignApi, JSONUtils.obj2json(body));
            if (result == null || !result.isSuccess()){
                log.error("查询单个无人机映射关系失败，body={}, result={}", body, result);
                return null;
            }
            return JSONUtils.json2obj(result.getData(), DirectControlUavParam.class);
        } catch (Exception e) {
            log.error("查询全量无人机映射关系失败: {}", e.getMessage());
        }
        return null;
    }
}
