package com.platform.service.clientmanage;

import com.google.common.util.concurrent.ThreadFactoryBuilder;
import lombok.extern.slf4j.Slf4j;

import java.util.concurrent.*;

/**
 * 在处理哈勃数据流程过程中处理和Cac的交互，减少数据传输的时延
 */
@Slf4j
public class CacRequestExecutor {

    private static final CacRequestExecutor INSTANCE = new CacRequestExecutor();
    private final ExecutorService executorService; // 根据实际需求调整线程数

    public CacRequestExecutor() {
        // 启动任务通知线程池
        ThreadFactory notifyThreadFactory = new ThreadFactoryBuilder()
                .setNameFormat("cacRequest" + "-%d")
                .setDaemon(true).build();
        int availableProcessors = Runtime.getRuntime().availableProcessors();
        int corePoolSize = availableProcessors * 2;
        int maxPoolSize = availableProcessors * 4;
        long keepAliveTime = 60L; // 60秒
        int queueCapacity = 200;

        executorService = new ThreadPoolExecutor(
                corePoolSize,
                maxPoolSize,
                keepAliveTime,
                TimeUnit.SECONDS,
                new ArrayBlockingQueue<>(queueCapacity),
                notifyThreadFactory
                // 默认使用AbortPolicy, 会抛异常
        );
        log.info("cacRequest线程池启动");
        // 注册 JVM 关闭挂钩，确保线程池在 JVM 退出时被关闭
        Runtime.getRuntime().addShutdownHook(new Thread(this::shutdown));
    }
    public static CacRequestExecutor getInstance() {
        return INSTANCE;
    }
    public Future<?> submit(Runnable task) {
        return INSTANCE.executorService.submit(task);
    }
    public void shutdown() {
        log.info("cacRequest线程池关闭...");
        executorService.shutdown();
        try {
            if (!executorService.awaitTermination(5, TimeUnit.SECONDS)) {
                executorService.shutdownNow();
            }
        } catch (InterruptedException e) {
            executorService.shutdownNow();
            Thread.currentThread().interrupt();
        }
    }
}

