package com.platform.service.clientmanage;


import com.platform.info.enums.ClientTypeEnum;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.util.AttributeKey;
import lombok.extern.slf4j.Slf4j;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j

/**
 * @Author : shiyi
 * @Date : 2023/12/21 17:23
 * @Description : 客户端管理
 */

public class ClientManager{
    public static final AttributeKey<BaseClient> CLIENT_ATTRIBUTE_KEY = AttributeKey.valueOf("client");
    /**
     *  客户端编号映射 编号→客户端对象
     */
    private static final Map<String, BaseClient> CLIENT_MAP = new ConcurrentHashMap<>();

    public static boolean isLogin(String clientSn){
        return null != getClient(clientSn);
    }

    /** 添加一个客户端对象并发送上线请求
     */
    public static void addAndOnline(BaseClient client) {
        client.getChannel().attr(CLIENT_ATTRIBUTE_KEY).set(client);
        boolean online = client.online();
        if (client instanceof HaborClient) {
            HaborClient haborClient = (HaborClient) client;
            log.info("[cac]{} 上线{}", haborClient, online ? "成功" : "失败");
        } else {
            log.info("{} {} 上线{}", client.getClientType().getInfo(), client.getSn(), online ? "成功" : "失败");
        }
        // 即使上线失败仍然添加到map中，虽然上线失败，但是通道并不关闭, 可用于地面站通信
        CLIENT_MAP.put(client.getSn(), client);
    }

    // /**
    //  * 更新客户端编号
    //  * @param client 指定客户端对象
    //  * @param newSn 需要更新的新编号
    //  */
    // public static void updateClientSn(BaseClient client, String newSn){
    //     // 参数检查
    //     if (client == null || newSn == null || newSn.isEmpty()) {
    //         return;
    //     }
    //     // 获取旧的sn
    //     String oldSn = client.getSn();
    //     // 如果新旧sn相同，无需更新
    //     if (oldSn.equals(newSn)) {
    //         return;
    //     }
    //     client.setSn(newSn);
    //     CLIENT_MAP.remove(oldSn);
    //     CLIENT_MAP.put(newSn, client);
    // }

    /** 移除客户端，并发送下线请求
     * @param client 需要移除的客户端
     */
    public static void removeAndOffline(BaseClient client) {
        remove(client);
        boolean offline = client.offline();
        log.info("[cac] {} {} 下线{}", client.getClientType().getInfo(), client.getSn(), offline ? "成功" : "失败");
        client = null;
    }

    /** 移除客户端，并解除其和channel的绑定
     */
    public static void remove(BaseClient client) {
        CLIENT_MAP.remove(client.getSn());
        Channel channel = client.getChannel();
        channel.attr(ClientManager.CLIENT_ATTRIBUTE_KEY).set(null);
        // channel.close(); // // TODO 2024/3/14: 是否要马上断开连接？
    }

    /**
     * 根据编号获取一个客户端对象, 编号不可为null
     */
    public static BaseClient getClient(@NotNull String clientSn) throws NullPointerException{
        return CLIENT_MAP.get(clientSn);
    }

    public static ArrayList<BaseClient> getClientGroup(ClientTypeEnum clientType) {
        ArrayList<BaseClient> clientList = new ArrayList<>();
        CLIENT_MAP.forEach((k, v) -> {
            if (v.getClientType() == clientType) {
                clientList.add(v);
            }
        });
        return clientList;
    }
    /** 根据Channel获取一个客户端对象
     */
    public static BaseClient getClient(Channel channel) {
        return channel.attr(CLIENT_ATTRIBUTE_KEY).get();
    }


    /**
     * 给指定客户端发送消息
     * @param clientType 客户端类型
     * @param channel 客户端channel
     * @param frameType 帧类型
     * @param data 数据内容
     * @return ChannelFuture
     */
    // public static ChannelFuture send(ClientTypeEnum clientType, Channel channel, byte frameType, byte[] data) {
    //     NettyMessage message = NettyMessage.build(clientType);
    //     if (message == null) {
    //         log.error("[send] 错误的客户端类型，无法发送消息");
    //         return null;
    //     }
    //     message.setHead(clientType.getWriteHead());
    //     message.setType(frameType);
    //     message.setLength((short) data.length);
    //     message.setContent(data);
    //
    //     return channel.writeAndFlush(message);
    // }
    //
    //
    // /** 发送消息给地面站
    //  */
    // public static ChannelFuture sendToGcs(Channel channel, byte frameType, byte[] data) {
    //     BoxMessage message = new BoxMessage();
    //
    //     message.setHead(ClientTypeEnum.GCS.getWriteHead());
    //     message.setType(frameType);
    //     message.setLength((short) data.length);
    //     message.setContent(data);
    //
    //     return channel.writeAndFlush(message);
    // }

}
