package com.platform.service.clientmanage;

import com.google.common.util.concurrent.ThreadFactoryBuilder;
import io.netty.util.HashedWheelTimer;
import io.netty.util.Timeout;
import io.netty.util.TimerTask;
import lombok.extern.slf4j.Slf4j;

import java.util.concurrent.*;

/**
 * CommandManager 负责集中管理所有指令的超时判断，使用 Netty 的 HashedWheelTimer 实现
 */
@Slf4j
public class CommandManager {
    private static final CommandManager INSTANCE = new CommandManager();

    // Netty的HashedWheelTimer，用于管理指令超时
    private final HashedWheelTimer timer;

    // 线程池用于处理通知任务，固定大小以限制并发数
    private final CacRequestExecutor cacRequestExecutor;


    private CommandManager() {
        // 初始化HashedWheelTimer，tickDuration和ticksPerWheel可以根据实际需求调整
        // tickDuration：一个刻度的时间间隔即时间精度, 每个刻度500毫秒，
        // ticksPerWheel：总的刻度数, 需要为2^N次方
        int tickDuration = 500;
        int tickPerWheel = 64;
        ThreadFactory wheelTimerThreadFactory = new ThreadFactoryBuilder()
                .setNameFormat("wheelTimer" + "-%d")
                .setDaemon(true).build();
        this.timer = new HashedWheelTimer(wheelTimerThreadFactory, tickDuration, TimeUnit.MILLISECONDS, tickPerWheel);

        cacRequestExecutor = CacRequestExecutor.getInstance();
        // 注册 JVM 关闭挂钩，确保线程池在 JVM 退出时被关闭
        Runtime.getRuntime().addShutdownHook(new Thread(this::shutdown));
    }

    public static CommandManager getInstance() {
        return INSTANCE;
    }

    /**
     * 添加一个待回报的指令记录，并设置超时任务
     *
     * @param record 指令记录
     */
    public void addCommandRecord(CommandRecord record) {
        // 创建并提交超时任务
        TimerTask task = timeout -> {
            if (record.getStatus() == CommandRecord.WAITING_RESULT) {
                record.setStatus(CommandRecord.FAILED);
                log.warn("[cac] {} {}指令超时: {}", record.getClient().shortInfo(), record.getCommandType().getInfo(), record.shortInfo());
                // 提交超时通知任务
                submitResultNotification(record, false);
            } else {
                log.warn("[cac] {} {}指令已完成，状态为{}: {}", record.getClient().shortInfo(), record.getCommandType().getInfo(), record.getStatus(), record.shortInfo());
            }
        };
        // 设置超时时间
        Timeout timeout = this.timer.newTimeout(task, record.getExpireMills(), TimeUnit.MILLISECONDS);
        record.setTimeout(timeout);
    }


    /**
     * 提交通知任务到线程池，处理超时和成功的通知
     *
     * @param record    指令记录
     * @param isSuccess 通知类型，true 表示成功，false 表示失败（超时）
     */
    public void submitResultNotification(CommandRecord record, boolean isSuccess) {
        try {
            cacRequestExecutor.submit(() -> notifyCommandResult(record, isSuccess));
        } catch (RejectedExecutionException e) {
            log.error("[cac] {} {}指令通知任务提交失败: {}", record.getClient().shortInfo(), record.getCommandType().getInfo(), record.shortInfo(), e);
        }
    }

    private void notifyCommandResult(CommandRecord record, boolean isSuccess) {
        try {
            HaborClient client = record.getClient();
            switch (record.getCommandType()) {
                case CONTROL:
                    record.notifyControlCommandResult(client.uavId, isSuccess);
                    break;
                case PARAM_BIND:
                case ROUTE_BIND:
                    record.notifyParamBindCommandResult(client.uavId, client.uavType, isSuccess);
                    break;
                case PARAM_QUERY:
                    record.notifyParamQueryCommandResult(client.uavId, client.uavType, isSuccess);
                    break;
                default:
                    log.warn("[cac] 未知的指令类型：{}", record.getCommandType());
            }
        } catch (Exception e) {
            log.error("[cac] {} 处理{}指令回报通知时发生异常: {}", record.getClient().shortInfo(), record.getCommandType().getInfo(), e.getMessage(), e);
        } finally {
            record.clearCommand();
        }
    }

    /**
     * 关闭管理器，释放资源
     */
    public void shutdown() {
        log.info("时间轮关闭...");
        this.timer.stop();
    }
}