package com.platform.service.clientmanage;


import com.platform.cac.CacHpApi;
import com.platform.cac.tcp.CacClient;
import com.platform.info.enums.ClientTypeEnum;
import com.platform.info.enums.UavTypeEnum;
import com.platform.util.*;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.PooledByteBufAllocator;
import io.netty.buffer.Unpooled;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.util.concurrent.Future;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @Author : shiyi
 * @Date : 2024/9/18 10:38
 * @Description : 飞控客户端
 */
@Slf4j
@Getter
public abstract class HaborClient extends BaseClient {
    // static HashSet<String> fkSnSet = new HashSet<>();
    protected UavTypeEnum uavType;
    protected byte fkId;
    protected String uavId;

    // 数据交换控制
    protected final AtomicBoolean needToSendCommonData = new AtomicBoolean(false); // 是否发送常发帧
    protected int commonDataFreq; // 默认发送频率, 每秒发送帧数, 以各型号子类为准
    protected volatile int targetPeriod;
    protected Future scheduledFuture;
    protected static final int MIN_SCHEDULE_DELAY = 1; // 最小调度延迟，避免过于频繁的调度
    protected static final int DEFAULT_COMMON_DATA_FREQ = 5; // 默认常发帧发送频率，每秒发送帧数
    // 数据帧内容
    protected AtomicUByte frameIdx = new AtomicUByte(); // 飞控包序号

    protected boolean resourceAllocated;
    protected DataByteBuf dataBufToSend; // 默认发送常发数据帧 ; // 实际数据发送定时任务发送的数据内容，发送时将引用以下dataBuf
    protected CommonDataBuf commonDataBuf; // 常发数据帧, 复用
    protected ControlDataBuf controlDataBuf; // 开关指令数据帧， 复用
    protected BindDataBuf bindDataBuf; // 参数装订指令数据帧， 复用
    protected QueryDataBuf queryDataBuf; // 参数查询指令数据帧， 复用
    protected RouteDataBuf routeDataBuf; // 航线装订指令数据帧， 复用
    protected static final int FRAME_SIZE = 32;  // 如果各型号不一样，该字段取消final，并在子类中初始化
    protected static final byte[] HEAD = new byte[]{(byte) 0xeb, (byte) 0x90};

    // 控制指令回报判断， 指令队列目前只允许一个指令，前一个指令回报或过期后才能发送下一个指令
    protected CommandRecord previousControl;
    // 装订指令回报判断
    protected CommandRecord previousParamBind;
    // 查询指令回报判断
    protected CommandRecord previousParamQuery;
    // 航线装订指令回报判断
    protected CommandQueueRecord previousRouteBind;

    // 遥测频率较高，转发中心指控的时候不在CacClient中进行协议类实例化，这里直接创建ByteBuf模板
    protected ByteBuf telemetryBufHeadToCac;
    protected int cacHeadLength; // 中心指控协议头长度, 见协议序号1-10
    protected AtomicUShort cacFrameIdx = new AtomicUShort(); // cac包序号
    protected long lastSendTime = 0;

    public HaborClient(String sn, Channel channel) {
        super(sn, channel);
        // channel绑定客户端
        channel.attr(ClientManager.CLIENT_ATTRIBUTE_KEY).setIfAbsent(this);
    }

    public static HaborClient createClient(UavTypeEnum uavType, String haborSn, Channel channel) {
        switch (uavType) {
            case FP981A:
                return new HaborClient981A(haborSn, channel);
            default:
                break;
        }
        return null;
    }

    @Override
    public String toString() {
        return getClass().getSimpleName() + "{" +
                "channel=" + channel.id() +
                ", sn='" + sn + '\'' +
                ", ip='" + ip + '\'' +
                ", port='" + port + '\'' +
                ", uavId='" + uavId + '\'' +
                ", fkId=" + fkId +
                ", uavType=" + uavType.getInfo() +
                '}';
    }

    public String shortInfo() {
        return "[sn='" + sn + '\'' +
                ", uavId='" + uavId + '\'' +
                ", fkId=" + fkId +
                ", uavType=" + uavType.getInfo() +
                ']';
    }


    @Override
    public boolean online() {
        // 仅当上线成功时才分配资源并进行和中心指控的通信
        boolean notifySuccess = CacHpApi.uavMasterControlNotify(uavId);
        if (notifySuccess) {
            resourceAllocate();
            startSendCommonData();
        }
        online = notifySuccess; // 资源分配完毕再确定为上线
        return online;
    }

    @Override
    public boolean offline() {
        // 仅当channel关闭，client被移除时调用，无论下线通知是否成功，均置为离线
        resourceRelease();
        stopSendCommonData();
        online = false;
        return CacHpApi.uavPowerOff(uavId);
    }
     @Override
    public boolean isOnline() {return online;}
    @Override
    public ClientTypeEnum getClientType() {
        return ClientTypeEnum.HABOR;
    }

    /**
     * 被管理时初始化
     */
    protected abstract void resourceAllocate();

    /**
     * 被移除管理时, 释放相关资源
     */
    protected void resourceRelease() {
        if (!resourceAllocated){
            return;
        }
        commonDataBuf.end();
        controlDataBuf.end();
        bindDataBuf.end();
        queryDataBuf.end();
        routeDataBuf.end();
        if (telemetryBufHeadToCac!= null && telemetryBufHeadToCac.refCnt() > 0) {
            telemetryBufHeadToCac.release(telemetryBufHeadToCac.refCnt());
        }
        previousControl.end();
        previousParamBind.end();
        previousParamQuery.end();
        previousRouteBind.end();
        resourceAllocated = false;
    }

    /**
     * 只通过get读数，不改变readerIndex和writerIndex, 也不可更改refCnt
     */
    public void process(ByteBuf msg) {
        if (msg.getByte(0) != HEAD[0] && msg.getByte(1) != HEAD[1]) {
            return; // 帧头错误，抛弃
        }
        if (!processInjectResponse(msg)) {
            // 如果消息未处理，则发送给中心指控
            sendToCac(msg.retain()); // 外面SimpleChannelInboundHandler会自动release msg, 所以这里需要retain一次
        }

    }

    /**
     * 注入回报帧处理
     */
    protected abstract boolean processInjectResponse(ByteBuf msg);

    /**
     * 发送常发帧
     */
    protected abstract void writeCommonData();

    /**
     * 向飞控发送控制指令
     */
    public abstract void writeControlCommand(byte commandCode, String uavControlUniId);

    /**
     * 向飞控发送参数装订指令
     */
    public abstract void writeParamBindCommand(byte commandCode, byte[] commandContent, String controlUniId);

    /**
     * 向飞控发送参数查询指令
     */
    public abstract void writeParamQueryCommand(byte commandCode, byte[] commandContent, String controlUniId);

    /**
     * 向飞控发送航线装订指令
     */
    public abstract void writeRouteBindCommand(byte commandCode, List<byte[]> routeInfoList, String controlUniId);


    // 常发数据帧
    protected void startSendCommonData() {
        if (scheduledFuture != null && !scheduledFuture.isCancelled()) {
            return;
        }

        needToSendCommonData.set(true);
        lastSendTime = System.currentTimeMillis();

        scheduledFuture = channel.eventLoop().submit(new Runnable() {
            @Override
            public void run() {
                try {
                    if (!needToSendCommonData.get() || !channel.isActive()) {
                        return;
                    }

                    long currentTime = System.currentTimeMillis();
                    long elapsed = currentTime - lastSendTime;

                    if (elapsed >= targetPeriod) {
                        // 批量发送逻辑，如果延迟太大，可以考虑补发
                        int missedFrames = (int) (elapsed / targetPeriod);
                        if (missedFrames > 2) { // 限制补发帧数
                            log.warn("常发帧延迟: {}ms", elapsed - targetPeriod);
                        }
                        writeCommonData();
                        lastSendTime = currentTime;

                        // 使用schedule而不是立即execute，给其他任务执行的机会
                        channel.eventLoop().schedule(this,
                                Math.max(MIN_SCHEDULE_DELAY, targetPeriod / 2),
                                TimeUnit.MILLISECONDS);
                    } else {
                        // 延迟剩余时间后再次调度
                        channel.eventLoop().schedule(this,
                                Math.max(MIN_SCHEDULE_DELAY, targetPeriod - elapsed),
                                TimeUnit.MILLISECONDS);
                    }
                } catch (Exception e) {
                    log.error("[cac] {} 常发帧调度异常: {}", shortInfo(), e.getMessage());
                    // 发生异常时，延迟一段时间后重试
                    if (needToSendCommonData.get() && channel.isActive()) {
                        channel.eventLoop().schedule(this, 500, TimeUnit.MILLISECONDS);
                    }
                }
            }
        });

        // 检查任务是否成功启动
        if (scheduledFuture != null && scheduledFuture.isCancellable()) {
            log.info("[cac] 开始向哈勃终端({})发送常发帧, 频率:{}帧/秒", shortInfo(), commonDataFreq);
        } else {
            log.info("[cac] 哈勃终端({})发送常发帧启动失败", shortInfo());
        }

    }


    protected void setCurrentSendData(DataByteBuf dataByteBuf) {
        setCurrentSendData(dataByteBuf, DEFAULT_COMMON_DATA_FREQ);
    }

    /**
     * 设置当前发送数据
     */
    protected void setCurrentSendData(DataByteBuf dataByteBuf, int freq) {
        dataByteBuf.resetSendCount();
        dataBufToSend = dataByteBuf;
        if (freq != commonDataFreq) {
            adjustSendFrequency(freq);
        }
    }

    // 调整常发帧发送频率
    protected void adjustSendFrequency(int newFreq) {
        if (newFreq <= 0) {
            throw new IllegalArgumentException("频率必须大于0");
        }
        commonDataFreq = newFreq;
        targetPeriod = 1000 / newFreq;
        log.info("[cac] {} 调整发送频率为 {} 帧/秒", shortInfo(), newFreq);
    }


    /**
     * 恢复常发帧数据
     */
    protected void switchToCommonData() {
        setCurrentSendData(commonDataBuf, DEFAULT_COMMON_DATA_FREQ);
        dataBufToSend.resetSendCount();
        log.info("[cac] {} 设置当前发送数据为常发数据帧", shortInfo());
    }


    // 停止定时任务常发数据帧
    protected void stopSendCommonData() {
        needToSendCommonData.set(false);
        if (scheduledFuture != null && !scheduledFuture.isCancelled()) {
            scheduledFuture.cancel(true);  // 取消任务并尝试中断
            scheduledFuture = null;
        }
        log.info("[cac] 停止向哈勃终端({})发送常发帧", shortInfo());
    }

    protected boolean isNeedToSendCommonData() {
        return needToSendCommonData.get();
    }

    /**
     * 暂停发送常发帧
     */
    protected void pauseSendCommonData() {
        needToSendCommonData.set(false);
        log.info("[cac] 暂停向哈勃终端({})发送常发帧", shortInfo());
    }

    /**
     * 恢复发送常发帧
     */
    protected void resumeSendCommonData() {
        needToSendCommonData.set(true);
        log.info("[cac] 恢复向哈勃终端({})发送常发帧", shortInfo());
    }


    /**
     * 发送飞控数据到中心指控
     *
     * @param srcBuf 原始遥测ByteBuf
     */
    private void sendToCac(ByteBuf srcBuf) {
        telemetryBufHeadToCac.writerIndex(cacHeadLength - 6); // 从包序号和包长度开始写入
        telemetryBufHeadToCac.writeShort(cacFrameIdx.getAndIncrement());
        telemetryBufHeadToCac.writeInt(srcBuf.readableBytes());
        ByteBuf telemetryBufToCac = Unpooled.wrappedBuffer(telemetryBufHeadToCac.retain(), srcBuf); // 遥测数据组合
        ChannelFuture future = CacClient.write(telemetryBufToCac);
        if (future != null) {
            // future.addListener((ChannelFuture f) -> {
            //     if (!f.isSuccess()) {
            //         telemetryBufToCac.release();
            //     }
            // });
        } else {
            telemetryBufToCac.release();
        }
    }


    /**
     * 给数据添加CRC16校验
     */
    protected void addCRC16(ByteBuf dataBuf) {
        ByteBuf slice = dataBuf.slice(2, 28);// 校验第3-30位，共28个字节
        dataBuf.writerIndex(30);
        dataBuf.writeBytes(CRCUtil.getCRCfromGcsTelemetryData(slice, 0, 28));
    }


    /**
     * 装订查询回报对象
     */
    public static abstract class UavQueryResultParam {
        public abstract String toJsonStr() throws IOException;
    }

    /**
     * Cac起飞参数查询查询回报，具体定义见各型号子类
     */
    public static abstract class UavTakeOffParamQueryResultParam extends UavQueryResultParam {
    }

    /**
     * Cac重量查询回报，具体定义见各型号子类
     */
    public static abstract class UavWeightQueryResultParam extends UavQueryResultParam {
    }


    /**
     * 数据发送对象
     */
    protected static class DataByteBuf {
        protected ByteBuf buf;
        private AtomicInteger sendCount = new AtomicInteger(0);

        public DataByteBuf(int frameSize) {
            buf = PooledByteBufAllocator.DEFAULT.buffer(frameSize);
        }

        public void end() {
            if (buf.refCnt() > 0) {
                buf.release(buf.refCnt());
            }
        }

        public void resetSendCount() {
            sendCount.set(0);
        }

        public int addSendCount() {
            return sendCount.incrementAndGet();
        }

        public int getSendCount() {
            return sendCount.get();
        }
    }

    /**
     * 常发帧
     */
    protected static class CommonDataBuf extends DataByteBuf {
        public CommonDataBuf(int frameSize) {
            super(frameSize);
        }
    }

    /**
     * 控制指令帧
     */
    protected static class ControlDataBuf extends DataByteBuf {
        public int needCount = 10; // 默认帧数

        public ControlDataBuf(int frameSize) {
            super(frameSize);
        }
    }

    /**
     * 数据装订帧
     */
    protected static class BindDataBuf extends DataByteBuf {
        public BindDataBuf(int frameSize) {
            super(frameSize);
        }
    }

    /**
     * 数据查询帧
     */
    protected static class QueryDataBuf extends DataByteBuf {
        public QueryDataBuf(int frameSize) {
            super(frameSize);
        }
    }

    /**
     * 航线数据帧
     */
    protected static class RouteDataBuf extends DataByteBuf {
        public RouteDataBuf(int frameSize) {
            super(frameSize);
        }
    }
}


