package com.platform.util;

import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import okhttp3.Request.Builder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @Author : shiyi
 * @Date : 2024/1/23 16:19
 * @Description : HttpClient工具类
 */
@Slf4j
public class HttpClientUtils {
    public static final String MEDIA_TYPE_JSON = "application/json; charset=utf-8";
    private static final OkHttpClient OKHTTP_CLIENT = new OkHttpClient.Builder()
            .connectTimeout(30, TimeUnit.SECONDS)
            .readTimeout(60, TimeUnit.SECONDS)
            .writeTimeout(60, TimeUnit.SECONDS)
            .retryOnConnectionFailure(true)
            .build();
    private static final OkHttpClient DONLOAD_OKHTTP_CLIENT = new OkHttpClient.Builder()
            .build();
    // public static <T> T sendGet(String url, Map<String, String> params, Class<T> responseClass) {
    //     MultiValueMap<String, String> queryParams = new LinkedMultiValueMap<>();
    //     if (params != null) {
    //         params.forEach(queryParams::add);
    //     }
    //     Mono<T> mono = WebClient.create(url).method(HttpMethod.GET)
    //             .uri(uriBuilder -> uriBuilder
    //                     .queryParams(queryParams)
    //                     .build()).retrieve().bodyToMono(responseClass);
    //     T responseBody;
    //     try {
    //         responseBody = mono.block();
    //     } catch (WebClientResponseException e) {
    //         log.error("Get 请求错误, {}", e.getMessage());
    //         return null;
    //     }
    //     return responseBody;
    // }
    public static <T> T sendGet(String url, Map<String, String> params, Class<T> responseClass) throws Exception {
        String jsonStr = sendGet(url, params);
        return JSONUtils.json2obj(jsonStr, responseClass);
    }
    public static String sendGet(String url, Map<String, String> params) throws IOException {
        StringBuilder stringBuilder = new StringBuilder();
        if (!CollectionUtils.isEmpty(params)) {
            for (String res : params.keySet()) {
                if (res == null || params.get(res) == null) {
                    continue;
                }
                if (StringUtils.isNotBlank(stringBuilder)) {
                    stringBuilder.append("&");
                } else {
                    stringBuilder.append("?");
                }
                try {
                    stringBuilder.append(String.format("%s=%s", res, URLEncoder.encode(params.get(res), "UTF-8")));
                } catch (UnsupportedEncodingException e) {
                    throw new RuntimeException(e);
                }
            }
        }
        Request request = new Builder()
                .url(url + stringBuilder)
                .build();

        try (Response response = OKHTTP_CLIENT.newCall(request).execute()) {
            if (!response.isSuccessful()) {
                throw new IOException("okhttp Unexpected code " + response);
            }
            ResponseBody body = response.body();
            if (body != null) {
                return body.string();
            } else {
                return null;
            }
        }
    }

    public static <T> T sendPost(String url, String json, Map<String, String> headers, Class<T> responseClass) throws Exception {
        String jsonStr = sendPost(url, json, headers);
        return JSONUtils.json2obj(jsonStr, responseClass);
    }

    public static <T> T sendPost(String url, String json, Class<T> responseClass) throws Exception {
        String jsonStr = sendPost(url, json, (Map) null);
        return JSONUtils.json2obj(jsonStr, responseClass);
    }

    public static String sendPost(String url, String json, Map<String, String> headers) throws IOException {
        RequestBody requestBody = RequestBody.create(MediaType.parse(MEDIA_TYPE_JSON), json);
        Builder requestBuilder = new Builder()
                .url(url)
                .post(requestBody);
        // 添加请求头
        if (!CollectionUtils.isEmpty(headers)) {
            for (Map.Entry<String, String> entry : headers.entrySet()) {
                requestBuilder.addHeader(entry.getKey(), entry.getValue());
            }
        }

        Request request = requestBuilder.build();
        try (Response response = OKHTTP_CLIENT.newCall(request).execute()) {
            if (!response.isSuccessful()) {
                throw new IOException("okhttp Unexpected code " + response);
            }
            ResponseBody body = response.body();
            if (body != null) {
                return body.string();
            } else {
                return null;
            }
        }
    }
    public static void downloadFileByUrl(String sourceUrl, String localPath) throws IOException {
        File file = new File(localPath);
        Request request = new Request.Builder()
                .url(sourceUrl)
                .build();

        try (Response response = DONLOAD_OKHTTP_CLIENT.newCall(request).execute()) {
            if (response.code() == 200) {
                assert response.body() != null;
                InputStream stream = response.body().byteStream();
                long dataBytes = Files.copy(stream, file.toPath(), StandardCopyOption.REPLACE_EXISTING);
                log.debug("文件{}下载完成，文件大小: {} bytes", localPath, dataBytes);
            } else {
                throw new RuntimeException("文件下载失败，Response：" + response);
            }
        }

    }

}
