package com.platform.util;


import com.fasterxml.jackson.core.type.TypeReference;
import io.netty.channel.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import javax.annotation.PostConstruct;
import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;

@Component
@Slf4j
public class SessionCache {
    public Map<String, Channel> tcpSession = new ConcurrentHashMap<>(16);

    public Map<String, String> addToSnMap = new ConcurrentHashMap<>(16);    // ip地址对应的设备编号
    public Map<String, String> snToAddMap = new ConcurrentHashMap<>(16);  //  设备编号对应的ip地址
    public Map<String, String> snToSnMap = new ConcurrentHashMap<>(16); //一对一映射关系
    public Map<String, String> snToMonitorMap = new ConcurrentHashMap<>(16);    //监控映射
    public Map<String, List<String>> mappingListMap = new ConcurrentHashMap<>(16);  //所有映射关系
    public Map<String, List<String>> snToListMap = new ConcurrentHashMap<>(16); //一对多映射关系
    public List<String> snList = new ArrayList<>();

    private static final String MAPPING_LIST_MAP_FILE = System.getProperty("user.dir") + File.separator + "mappingListMap.json";



    public Optional<Channel>  findTargetChannel(String sourceAddr) {
        String sSn = addToSnMap.get(sourceAddr);
        String tSn = snToSnMap.get(sSn);
        if (tSn == null) {
            log.warn("找不到" + sourceAddr + "的目标映射;" +
                    "addToSnMap = " + addToSnMap + "; " +
                    "snToSnMap = " + snToSnMap);
            return Optional.empty();
        }

        String targetAddr = snToAddMap.get(tSn);
        if (targetAddr == null) {
            log.warn("目标映射" + tSn + "的地址找不到; " +
                    "snToAddMap = " + snToAddMap);
            return Optional.empty();
        }

        Channel c = tcpSession.get(targetAddr);
        if (c == null) {
            log.warn("目标地址" + targetAddr + "的session channel找不到; " +
                    "tcpSession = " + tcpSession);
            return Optional.empty();
        }

        return Optional.of(c);
    }

    public Optional<Channel>  findMonitorChannel(String sourceAddr) {
        String sSn = addToSnMap.get(sourceAddr);
        String tSn = snToMonitorMap.get(sSn);
        if (tSn == null) {
            log.warn("找不到" + sourceAddr + "的目标映射;" +
                    "addToSnMap = " + addToSnMap + "; " +
                    "snToMonitorMap = " + snToMonitorMap);
            return Optional.empty();
        }

        String targetAddr = snToAddMap.get(tSn);
        if (targetAddr == null) {
            log.warn("目标映射" + tSn + "的地址找不到; " +
                    "snToAddMap = " + snToAddMap);
            return Optional.empty();
        }

        Channel c = tcpSession.get(targetAddr);
        if (c == null) {
            log.warn("目标地址" + targetAddr + "的session channel找不到; " +
                    "tcpSession = " + tcpSession);
            return Optional.empty();
        }

        return Optional.of(c);
    }

    public List<Channel> findManyTargetChannel(String sSn) {
        List<Channel> channel = new ArrayList<>();
        List<String> tSn = snToListMap.get(sSn);

        if (!ObjectUtils.isEmpty(tSn)) {
            tSn.forEach(s -> {
                String targetAddr = snToAddMap.get(s);
                if (targetAddr != null) {
                    Channel c = tcpSession.get(targetAddr);
                    if (c == null) {
                        log.warn("targetAddr " + targetAddr + " channel is null" + ", tcpSession = " + tcpSession);
                    } else {
                        channel.add(c);
                    }
                }
            });
        }

        return channel;
    }

    /**根据sn获取对应需要转发的channel*/
    public List<Channel> findMappingTargetChannel(String sSn) {
        List<Channel> channel = new ArrayList<>();
        List<String> tSn = mappingListMap.get(sSn);                  // 设备的映射关系

        if (!ObjectUtils.isEmpty(tSn)) {
            tSn.forEach(s -> {                                   //
                String targetAddr = snToAddMap.get(s);           // 根据设备编号找到ip地址
                if (targetAddr != null) {
                    Channel c = tcpSession.get(targetAddr);     // 根据ip找到通道
                    if (c == null) {
                        log.warn("targetAddr " + targetAddr + " channel is null" + ", tcpSession = " + tcpSession);
                    } else {
                        channel.add(c);
                    }
                }
            });
        }

        return channel;

    }


    /**
     * channel找ip地址
     */

    public String findSnByChannel(Channel channel){
        String fk = null;
        for(String key: tcpSession.keySet()){
            if(tcpSession.get(key).equals(channel)){
                fk = key;
            }
        }
        return fk;
    }


    // mappingListMap 保存到本地文件
    @PostConstruct
    public void init() {
        loadMappingListMapFromFile();
    }

    /**
     * 获取 mappingListMap
     */
    public Map<String, List<String>> getMappingListMap() {
        return mappingListMap;
    }

/**
     * 加载持久化的 mappingListMap
     */
    private void loadMappingListMapFromFile() {
        File file = new File(MAPPING_LIST_MAP_FILE);
        if (!file.exists()) {
            log.info("[config] haborSn-gcsMac映射关系文件不存在，初始化为空");
            return;
        }
        try (BufferedReader reader = new BufferedReader(new FileReader(file))) {
            StringBuilder jsonBuilder = new StringBuilder();
            String line;
            while ((line = reader.readLine()) != null) {
                jsonBuilder.append(line);
            }
            String json = jsonBuilder.toString();
            if (!json.isEmpty()) {
                Map<String, List<String>> loadedMap = JSONUtils.json2obj(json, new TypeReference<Map<String, List<String>>>() {});
                if (loadedMap != null) {
                    this.mappingListMap.putAll(loadedMap);
                    log.info("[config] 成功加载 haborSn-gcsMac 映射关系: {}", mappingListMap);
                }
            }
        } catch (IOException e) {
            log.error("[config] 加载 haborSn-gcsMac 数据 {} 失败：" , MAPPING_LIST_MAP_FILE,e);
        }
    }

    /**
     * 保存 mappingListMap 到文件
     */
    public synchronized void saveMappingListMapToFile() {
        File file = new File(MAPPING_LIST_MAP_FILE);
        try (BufferedWriter writer = new BufferedWriter(new FileWriter(file))) {
            String json = JSONUtils.obj2json(this.mappingListMap);
            writer.write(json);
            log.info("[config] 成功保存 haborSn-gcsMac 映射关系到文件 {}", MAPPING_LIST_MAP_FILE);
        } catch (IOException e) {
            log.error("[config] 保存 haborSn-gcsMac 数据 {} 失败：" , MAPPING_LIST_MAP_FILE,e);
        }
    }
}
